import click
from notion.client import NotionClient

from kindle2notion.exporting import export_to_notion
from kindle2notion.parsing import parse_raw_clippings_text
from kindle2notion.reading import read_raw_clippings


@click.command()
@click.argument("notion_token")
@click.argument("notion_table_id")
@click.argument("clippings_file_path")
@click.option(
    "--enable_highlight_date",
    default=True,
    help='Set to False if you don\'t want to see the "Date Added" information in Notion.',
)
@click.option(
    "--enable_book_cover",
    default=True,
    help="Set to False if you don't want to store the book cover in Notion.",
)
def main(
    notion_token,
    notion_table_id,
    clippings_file_path,
    enable_highlight_date,
    enable_book_cover,
):

    notion_client = NotionClient(token_v2=notion_token)
    notion_collection_view = notion_client.get_collection_view(notion_table_id)

    if len(notion_collection_view.parent.views) > 0:
        print("Notion page is found. Analyzing clippings file...")
        all_clippings = read_raw_clippings(clippings_file_path)
        books = parse_raw_clippings_text(all_clippings)
        export_to_notion(
            books,
            enable_highlight_date,
            enable_book_cover,
            notion_token,
            notion_table_id,
        )
        print("Transfer complete... Exiting script...")


if __name__ == "__main__":
    main()



from datetime import datetime
from typing import Dict, List, Tuple

from dateparser import parse
from notion.block import TextBlock
from notion.client import NotionClient
from notion.collection import NotionDate
from requests import get

NO_COVER_IMG = "https://via.placeholder.com/150x200?text=No%20Cover"
ITALIC = "*"
BOLD = "**"

# TODO: Refactor this module


def export_to_notion(
    books: Dict,
    enable_highlight_date: bool,
    enable_book_cover: bool,
    notion_token: str,
    notion_table_id: str,
) -> None:
    print("Initiating transfer...\n")

    for title in books:
        book = books[title]
        author = book["author"]
        highlights = book["highlights"]
        highlight_count = len(highlights)
        (
            aggregated_text_from_highlights,
            last_date,
        ) = _prepare_aggregated_text_for_one_book(highlights, enable_highlight_date)
        message = _add_book_to_notion(
            title,
            author,
            highlight_count,
            aggregated_text_from_highlights,
            last_date,
            notion_token,
            notion_table_id,
            enable_book_cover,
        )
        if message != "None to add":
            print("✓", message)


def _prepare_aggregated_text_for_one_book(
    highlights: List, enable_highlight_date: bool
) -> Tuple[str, str]:
    aggregated_text = ""
    for highlight in highlights:
        text = highlight[0]
        page = highlight[1]
        location = highlight[2]
        date = highlight[3]
        isNote = highlight[4]
        if isNote == True:
            aggregated_text += BOLD + "Note: " + BOLD

        aggregated_text += text + "\n("
        if page != "":
            aggregated_text += ITALIC + "Page: " + page + ITALIC + "  "
        if location != "":
            aggregated_text += ITALIC + "Location: " + location + ITALIC + "  "
        if enable_highlight_date and (date != ""):
            aggregated_text += ITALIC + "Date Added: " + date + ITALIC

        aggregated_text = aggregated_text.strip() + ")\n\n"
    last_date = date
    return aggregated_text, last_date


def _add_book_to_notion(
    title: str,
    author: str,
    highlight_count: int,
    aggregated_text: str,
    last_date: str,
    notion_token: str,
    notion_table_id: str,
    enable_book_cover: bool,
) -> str:
    notion_client = NotionClient(token_v2=notion_token)
    notion_collection_view = notion_client.get_collection_view(notion_table_id)
    notion_collection_view_rows = notion_collection_view.collection.get_rows()

    title_exists = False
    if notion_collection_view_rows:
        for row in notion_collection_view_rows:
            if title == row.title and author == row.author:
                title_exists = True
                row = row

                if row.highlights is None:
                    row.highlights = 0  # to initialize number of highlights as 0
                elif row.highlights == highlight_count:  # if no change in highlights
                    return "None to add"

    title_and_author = title + " (" + str(author) + ")"
    print(title_and_author)
    print("-" * len(title_and_author))

    if not title_exists:
        row = notion_collection_view.collection.add_row()
        row.title = title
        row.author = author
        row.highlights = 0

        if enable_book_cover:
            if row.cover is None:
                result = _get_book_cover_uri(row.title, row.author)
            if result is not None:
                row.cover = result
                print("✓ Added book cover")
            else:
                row.cover = NO_COVER_IMG
                print(
                    "× Book cover couldn't be found. "
                    "Please replace the placeholder image with the original book cover manually."
                )

    parent_page = notion_client.get_block(row.id)

    # For existing books with new highlights to add
    for all_blocks in parent_page.children:
        all_blocks.remove()
    parent_page.children.add_new(TextBlock, title=aggregated_text)
    diff_count = highlight_count - row.highlights
    row.highlights = highlight_count
    row.last_highlighted = NotionDate(parse(last_date))
    row.last_synced = NotionDate(datetime.now())
    message = str(diff_count) + " notes / highlights added successfully\n"
    return message


def _get_book_cover_uri(title: str, author: str):
    req_uri = "https://www.googleapis.com/books/v1/volumes?q="

    if title is None:
        return
    req_uri += "intitle:" + title

    if author is not None:
        req_uri += "+inauthor:" + author

    response = get(req_uri).json().get("items", [])
    if len(response) > 0:
        for x in response:
            if x.get("volumeInfo", {}).get("imageLinks", {}).get("thumbnail"):
                return (
                    x.get("volumeInfo", {})
                    .get("imageLinks", {})
                    .get("thumbnail")
                    .replace("http://", "https://")
                )
    return



# import unicodedata
from pathlib import Path


def read_raw_clippings(clippings_file_path: Path) -> str:
    raw_clippings_text = open(clippings_file_path, "r", encoding="utf-8-sig").read()
    raw_clippings_text = raw_clippings_text.encode("ascii", errors="ignore").decode()
    return raw_clippings_text



import re
from typing import Dict, List, Tuple

from dateparser import parse

ACADEMIC_TITLES = [
    "A.A.",
    "A.S.",
    "A.A.A.",
    "A.A.S.",
    "A.B.",
    "A.D.N.",
    "A.M.",
    "A.M.T.",
    "C.E.",
    "Ch.E.",
    "D.A.",
    "D.A.S.",
    "D.B.A.",
    "D.C.",
    "D.D.",
    "D.Ed.",
    "D.L.S.",
    "D.M.D.",
    "D.M.S.",
    "D.P.A.",
    "D.P.H.",
    "D.R.E.",
    "D.S.W.",
    "D.Sc.",
    "D.V.M.",
    "Ed.D.",
    "Ed.S.",
    "E.E.",
    "E.M.",
    "E.Met.",
    "I.E.",
    "J.D.",
    "J.S.D.",
    "L.H.D.",
    "Litt.B.",
    "Litt.M.",
    "LL.B.",
    "LL.D.",
    "LL.M.",
    "M.A.",
    "M.Aero.E.",
    "M.B.A.",
    "M.C.S.",
    "M.D.",
    "M.Div.",
    "M.E.",
    "M.Ed.",
    "M.Eng.",
    "M.F.A.",
    "M.H.A.",
    "M.L.S.",
    "M.Mus.",
    "M.N.",
    "M.P.A.",
    "M.S.",
    "M.S.Ed.",
    "M.S.W.",
    "M.Th.",
    "Nuc.E.",
    "O.D.",
    "Pharm.D.",
    "Ph.B.",
    "Ph.D.",
    "S.B.",
    "Sc.D.",
    "S.J.D.",
    "S.Sc.D.",
    "Th.B.",
    "Th.D.",
    "Th.M.",
]

DELIMITERS = ["; ", " & ", " and "]


def parse_raw_clippings_text(raw_clippings_text: str) -> Dict:
    raw_clippings_list = raw_clippings_text.split("==========")
    print(f"Found {len(raw_clippings_list)} notes and highlights.\n")

    books = {}
    passed_clippings_count = 0

    for raw_clipping in raw_clippings_list:
        raw_clipping_list = raw_clipping.strip().split("\n")

        if _is_valid_clipping(raw_clipping_list):
            author, title = _parse_author_and_title(raw_clipping_list)
            page, location, date, isNote = _parse_page_location_date_and_note(
                raw_clipping_list
            )
            highlight = raw_clipping_list[3]

            books = _add_parsed_items_to_books_dict(
                books, title, author, highlight, page, location, date, isNote
            )
        else:
            passed_clippings_count += 1

    print(f"× Passed {passed_clippings_count} bookmarks or unsupported clippings.\n")
    return books


def _is_valid_clipping(raw_clipping_list: List) -> bool:
    return len(raw_clipping_list) >= 3


def _parse_author_and_title(raw_clipping_list: List) -> Tuple[str, str]:
    author, title = _parse_raw_author_and_title(raw_clipping_list)
    author, title = _deal_with_exceptions_in_author_name(author, title)
    title = _deal_with_exceptions_in_title(title)
    return author, title


def _parse_page_location_date_and_note(
    raw_clipping_list: List,
) -> Tuple[str, str, str, bool]:
    second_line = raw_clipping_list[1]
    second_line_as_list = second_line.strip().split(" | ")
    page = location = date = ""
    isNote = False

    for element in second_line_as_list:
        element = element.lower()
        if "note" in element:
            isNote = True
        if "page" in element:
            page = element[element.find("page") :].replace("page", "").strip()
        if "location" in element:
            location = (
                element[element.find("location") :].replace("location", "").strip()
            )
        if "added on" in element:
            date = parse(
                element[element.find("added on") :].replace("added on", "").strip()
            )
            date = date.strftime("%A, %d %B %Y %I:%M:%S %p")

    return page, location, date, isNote


def _add_parsed_items_to_books_dict(
    books: Dict,
    title: str,
    author: str,
    highlight: str,
    page: str,
    location: str,
    date: str,
    isNote: bool,
) -> Dict:
    if title not in books:
        books[title] = {"author": author, "highlights": []}
    books[title]["highlights"].append((highlight, page, location, date, isNote))
    return books


def _parse_raw_author_and_title(raw_clipping_list: List) -> Tuple[str, str]:
    author = ""
    title = raw_clipping_list[0]

    if re.findall(r"\(.*?\)", raw_clipping_list[0]):
        author = (re.findall(r"\(.*?\)", raw_clipping_list[0]))[-1]
        author = author.removeprefix("(").removesuffix(")")
    else:
        print(
            "No author found. You can manually add the Author details in the Notion database."
        )

    title = raw_clipping_list[0].replace(author, "").strip().replace(" ()", "")

    return author, title


def _deal_with_exceptions_in_author_name(author: str, title: str) -> Tuple[str, str]:
    if "(" in author:
        author = author + ")"
        title = title.removesuffix(")")

    if ", " in author and all(x not in author for x in DELIMITERS):
        if (author.split(", "))[1] not in ACADEMIC_TITLES:
            author = " ".join(reversed(author.split(", ")))

    if "; " in author:
        authorList = author.split("; ")
        author = ""
        for ele in authorList:
            author += " ".join(reversed(ele.split(", "))) + ", "
        author = author.removesuffix(", ")
    return author, title


def _deal_with_exceptions_in_title(title: str) -> str:
    if ", The" in title:
        title = "The " + title.replace(", The", "")
    return title



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

with open("requirements.txt", "r", encoding="utf-8") as f:
    requirements = f.read()

with open("requirements-dev.txt", "r", encoding="utf-8") as f:
    requirements_dev = f.read()

setup(
    name="kindle2notion",
    version="0.1.8",
    author="Jeffrey Jacob",
    author_email="jeffreysamjacob@gmail.com",
    description="Export all the clippings from your Kindle device to a database in Notion.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/paperboi/kindle2notion",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    install_requires=requirements,
    extras_require={"dev": requirements_dev},
    python_requires=">=3.9",
    entry_points={
        "console_scripts": [
            "kindle2notion = kindle2notion.__main__:main",
        ],
    },
)
